/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005-2010, Red Hat Middleware LLC, and individual contributors
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */


package org.jboss.messaging.util;

import java.math.BigInteger;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.spec.SecretKeySpec;

/**
 * A SecurityUtil a util class used to encode/decode passwords
 * implementation copied from org.jboss.resource.security.SecureIdentityLoginModule
 * 
 * @author <a href="mailto:hgao@redhat.com">Howard Gao</a>
 * 
 * Created May 12, 2010 9:30:11 PM
 *
 *
 */
public final class SecurityUtil
{

   private static final byte[] KEY_BYTES = "sucker default".getBytes();
   
   public static String encode(String secret) throws NoSuchPaddingException,
                                             NoSuchAlgorithmException,
                                             InvalidKeyException,
                                             BadPaddingException,
                                             IllegalBlockSizeException
   {
      SecretKeySpec key = new SecretKeySpec(KEY_BYTES, "Blowfish");

      Cipher cipher = Cipher.getInstance("Blowfish");
      System.out.println("key len: " + KEY_BYTES.length + " length max: " + Cipher.getMaxAllowedKeyLength("Blowfish"));
      cipher.init(Cipher.ENCRYPT_MODE, key);
      byte[] encoding = cipher.doFinal(secret.getBytes());
      BigInteger n = new BigInteger(encoding);
      return n.toString(16);
   }

   public static char[] decode(String secret) throws NoSuchPaddingException,
                                             NoSuchAlgorithmException,
                                             InvalidKeyException,
                                             BadPaddingException,
                                             IllegalBlockSizeException
   {
      SecretKeySpec key = new SecretKeySpec(KEY_BYTES, "Blowfish");

      BigInteger n = new BigInteger(secret, 16);
      byte[] encoding = n.toByteArray();

      // JBAS-3457: fix leading zeros
      if (encoding.length % 8 != 0)
      {
         int length = encoding.length;
         int newLength = ((length / 8) + 1) * 8;
         int pad = newLength - length; // number of leading zeros
         byte[] old = encoding;
         encoding = new byte[newLength];
         for (int i = old.length - 1; i >= 0; i--)
         {
            encoding[i + pad] = old[i];
         }
      }

      Cipher cipher = Cipher.getInstance("Blowfish");
      cipher.init(Cipher.DECRYPT_MODE, key);
      byte[] decode = cipher.doFinal(encoding);
      return new String(decode).toCharArray();
   }

   /** Main entry point to encrypt a password using the hard-coded pass phrase
   *
   * @param args - [0] = the password to encode
   * @throws Exception
   */
   public static void main(String[] args) throws Exception
   {
      String encode = encode(args[0]);
      System.out.println("Encoded password: " + encode);
   }
}
