/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005-2009, Red Hat Middleware LLC, and individual contributors
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.test.messaging.jms;

import org.jboss.jms.client.container.ThreadValveCounter;

import junit.framework.TestCase;

/**
 * A ThreadValveCounterTest
 *
 * @author <a href="mailto:hgao@redhat.com">Howard Gao</a>
 * 
 * Created Dec 9, 2009 5:46:31 PM
 *
 */
public class ThreadValveCounterTest extends TestCase
{

   public void testHangCondition1()
   {
      SimpleValveInterceptor interceptor = new SimpleValveInterceptor();
      SharedTestObject obj = new SharedTestObject(interceptor);
      obj.invoke(4);
   }

   public void testHangCondition2()
   {
      SimpleValveInterceptor interceptor = new SimpleValveInterceptor();
      SharedTestObject obj = new SharedTestObject(interceptor);

      ValveTestThread thr1 = new ValveTestThread(obj, 1);
      thr1.start();

      ValveTestThread thr2 = new ValveTestThread(obj, 0);
      thr2.start();

      try
      {
         thr1.join();
         thr2.join();
      }
      catch (InterruptedException e)
      {
      }

   }

   public void testHangCondition3()
   {
      SimpleValveInterceptor interceptor = new SimpleValveInterceptor();
      SharedTestObject obj = new SharedTestObject(interceptor);

      ValveTestThread thr1 = new ValveTestThread(obj, 1);
      thr1.start();

      ValveTestThread thr2 = new ValveTestThread(obj, 0);
      thr2.start();

      ValveTestThread thr3 = new ValveTestThread(obj, 4);
      thr3.start();

      try
      {
         thr1.join();
         thr2.join();
         thr3.join();
      }
      catch (InterruptedException e)
      {
      }

   }

   public static class ValveTestThread extends Thread
   {
      private SharedTestObject target;

      private int mthd;

      public ValveTestThread(SharedTestObject obj, int m)
      {
         target = obj;
         mthd = m;
      }

      public void run()
      {
         target.invoke(mthd);
      }
   }

   public static class SimpleValveInterceptor
   {
      private ThreadValveCounter counter = new ThreadValveCounter();

      private volatile boolean closed = false;

      public void invoke(SharedTestObject object, int method)
      {
         if (closed)
            return;

         if (method == SharedTestObject.METHOD_CLOSE)
         {
            waitingForClose();
            return;
         }

         counter.pushThread();
         object.invokeMethod(method);
         counter.popThread();
      }

      protected void waitingForClose()
      {
         if (closed)
         {
            return;
         }

         counter.waitForCompletion();

         closed = true;
      }
   }

   public static class SharedTestObject
   {
      public static final int METHOD_CLOSE = 0;

      public static final int METHOD_1 = 1;

      public static final int METHOD_2 = 2;

      public static final int METHOD_3 = 3;

      public static final int METHOD_4 = 4;

      private SimpleValveInterceptor inter;

      public SharedTestObject(SimpleValveInterceptor interceptor)
      {
         inter = interceptor;
      }

      public void invoke(int m)
      {
         inter.invoke(this, m);
      }

      public void invokeMethod(int m)
      {
         switch (m)
         {
            case METHOD_CLOSE:
               close();
               break;
            case METHOD_1:
               method1();
               break;
            case METHOD_2:
               method2();
               break;
            case METHOD_3:
               method3();
               break;
            case METHOD_4:
               method4();
               break;
         }
      }

      public void close()
      {

      }

      public void method1()
      {
         try
         {
            Thread.sleep(5000);
         }
         catch (InterruptedException e)
         {
            e.printStackTrace();
         }
      }

      public void method2()
      {
      }

      public void method3()
      {
      }

      public void method4()
      {
         this.invoke(0);
      }
   }
}
